/* 
 * $TSUKUBA_Release: Omni Compiler Version 0.9.2 $
 * $TSUKUBA_Copyright:
 *  Copyright (C) 2010-2014 University of Tsukuba, 
 *  	      2012-2014  University of Tsukuba and Riken AICS
 *  
 *  This software is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License version
 *  2.1 published by the Free Software Foundation.
 *  
 *  Please check the Copyright and License information in the files named
 *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
 *  Software release kit.
 *  
 *  * The specification of XcalableMP has been designed by the XcalableMP
 *    Specification Working Group (http://www.xcalablemp.org/).
 *  
 *  * The development of this software was partially supported by "Seamless and
 *    Highly-productive Parallel Programming Environment for
 *    High-performance computing" project funded by Ministry of Education,
 *    Culture, Sports, Science and Technology, Japan.
 *  $
 */

/*---------------------------------------*\
 * Adapter about array shape of Fortran
\*---------------------------------------*/

package exc.object;
import exc.block.Block;
import java.util.*;
import java.math.BigInteger;

public class Fshape
{
  private Block block = null;

  private int _rank;
  private Xobject[] _lbound;
  private Xobject[] _ubound;
  private Xobject[] _extent;

  //
  // constructors
  //
  public Fshape(FindexRange findexRange)
  {
    _rank = findexRange.getFrank();
    _lbound = findexRange.getLbounds();
    _ubound = findexRange.getUbounds();
    _extent = findexRange.getExtents();
  }

  public Fshape(FarrayType farrayType, Block block)
  {
    this(farrayType.getFindexRange(block));
    this.block = block;
  }

  public Fshape(Ident ident)
  {
    _rank = ident.getFrank();

    if (_rank == 0) {
      _lbound = null;
      _ubound = null;
      _extent = null;
      return;
    }

    Xtype type = ident.Type();
    switch (type.getKind()) {
    case Xtype.BASIC:
      _restrict("Ident of Xtype.BASIC not supported. ident=" + ident);
      break;

    case Xtype.F_ARRAY:
      Fshape fshape = new Fshape((FarrayType)type, block);
      _lbound = fshape.lbounds();
      _ubound = fshape.ubounds();
      _extent = fshape.extents();
      break;

    case Xtype.F_COARRAY:
      _restrict("Ident of Xtype.F_COARRAY not supported. ident=" + ident);
      // type.coshape might be useful.
      break;

    default:
      /* illegal kind of Xtype */
      _error("Found illegal type of ident. ident=" + ident);
      return;
    }
  }

  public Fshape(XobjString xobjString, Block block)
  {
    this(xobjString.findIdent(block));
  }
    

  public Fshape(Xobject xobj)
  {
    this(xobj, null);
  }

  public Fshape(Xobject xobj, Block block)
  {
    this.block = block;

    _error("unexpected Xobject. xobj=" + xobj);
  }

  //
  // access methods
  //
  public Xobject[] lbounds()
  {
    return _lbound;
  }
  public Xobject lbound(int dim)
  {
    return _lbound[dim];
  }

  public Xobject[] ubounds()
  {
    return _ubound;
  }
  public Xobject ubound(int dim)
  {
    return _ubound[dim];
  }

  public Xobject[] extents()
  {
    return _extent;
  }
  public Xobject extent(int dim)
  {
    return _extent[dim];
  }


  /*************************************************\
   * error handler
  \*************************************************/
  private static void _info(String msg) {
    System.err.println("[XMP Fshape] INFO: " + msg);
  }

  private static void _warn(String msg) {
    System.err.println("[XMP Fshape] WARNING: " + msg);
  }

  private static void _error(String msg) {
    System.err.println("[XMP Fshape] ERROR: " + msg);
  }

  private static void _restrict(String msg) {
    System.err.println("[XMP Fshape] RESTRICTION: " + msg);
  }

}

